@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # @Author : 厦门理工计算机学院 王大寒/(PRIU)福建省模式识别与图像理解重点实验室
-# @Document : https://www.yuque.com/code4101/python/hesuan
+# @Document : https://www.yuque.com/xlpr/doai/hesuan
 # @Email  : (陈)877362867@qq.com
 # @Address: 厦门理工综合楼1905
-# @Date   : 2022/08/28
+# @Date   : 2022/08/31
 
 import datetime
 import json
@@ -19,6 +19,9 @@ import pandas as pd
 import requests
 from tqdm import tqdm
 
+import xlrd2
+import cv2
+
 from PyQt5.QtCore import QSize, Qt, QUrl
 from PyQt5.QtGui import QFont, QIcon, QDesktopServices
 from PyQt5.QtWidgets import (QApplication, QMainWindow, QLabel, QGridLayout, QWidget,
@@ -26,7 +29,7 @@ from PyQt5.QtWidgets import (QApplication, QMainWindow, QLabel, QGridLayout, QWi
 
 from pyxllib.file.xlsxlib import openpyxl
 from pyxllib.gui.qt import get_input_widget
-from pyxllib.xl import TicToc, XlPath, matchpairs
+from pyxllib.xl import TicToc, XlPath, matchpairs, get_etag
 
 from pyxlpr.ai.clientlib import XlAiClient
 
@@ -35,21 +38,53 @@ class PPOCR:
     def __init__(self):
         self.xlapi = XlAiClient()
         self.xlapi.login_priu('14VITkixP7fD#)[')
+        # 本地缓存，已经识别过的，不重复提交服务器。
+        self.cache = self.read_cache()  # etag: result_attrs
+
+    def read_cache(self):
+        """ 每个月的结果会在本地做缓存 """
+        f = XlPath.tempdir() / datetime.date.today().strftime('hesuan%Y%m.pkl')
+        if f.is_file():
+            return f.read_pkl()
+        else:
+            return {}
+
+    def write_cache(self):
+        f = XlPath.tempdir() / datetime.date.today().strftime('hesuan%Y%m.pkl')
+        f.write_pkl(self.cache)
+
+    def get_image_buffer(self, f):
+        # 核酸识别的图片大概率不需要很清晰的图片，每张图文件大小可以控制在300kb以内的jpg文件
+        # ratio用不到，api接口只会返回关键类别字典值
+        buffer, ratio = self.xlapi.adjust_image(f, limit_b64buffer_size=500 * 1024, b64encode=True)
+        return buffer.decode()
 
     def parse_light(self, f):
-        return self.xlapi.priu_api(f'hesuan/parse_light', f)
+        xlapi = self.xlapi
+        etag = 'parse_light,' + get_etag(XlPath(f).read_bytes())
+        if etag not in self.cache:
+            data = {'image': self.get_image_buffer(f)}
+            r = requests.post(f'{xlapi._priu_host}/api/hesuan/parse_light',
+                              json.dumps(data), headers=xlapi._priu_header)
+            attrs = json.loads(r.text)
+            if 'xlapi' in attrs:
+                del attrs['xlapi']
+            self.cache[etag] = attrs
+        return self.cache[etag]
 
     def parse_multi_layout_light(self, files):
         xlapi = self.xlapi
-        data = {}
-        data['images'] = [xlapi._priu_read_image(x) for x in files]
-        r = requests.post(f'{xlapi._priu_host}/api/hesuan/parse_multi_layout_light',
-                          json.dumps(data), headers=xlapi._priu_header)
-        attrs = json.loads(r.text)
-        # print(attrs)
-        if 'xlapi' in attrs:
-            del attrs['xlapi']
-        return attrs
+        etag = 'parse_multi_layout_light,' + ','.join([get_etag(XlPath(f).read_bytes()) for f in files])
+        if etag not in self.cache:
+            data = {'images': [self.get_image_buffer(x) for x in files]}
+            r = requests.post(f'{xlapi._priu_host}/api/hesuan/parse_multi_layout_light',
+                              json.dumps(data), headers=xlapi._priu_header)
+            attrs = json.loads(r.text)
+            # print(attrs)
+            if 'xlapi' in attrs:
+                del attrs['xlapi']
+            self.cache[etag] = attrs
+        return self.cache[etag]
 
 
 class Hesuan:
@@ -301,7 +336,12 @@ class Hesuan:
                 if f.name == 'xmut.jpg':
                     # 有的人没设目录，直接在根目录全量查找了，此时需要过滤掉我的一个资源图片
                     continue
-                attrs = self.ppocr.parse_light(f)
+                try:
+                    attrs = self.ppocr.parse_light(f)
+                except Exception as e:
+                    print(f'{f.as_posix()} 图片有问题，跳过未处理。')
+                    continue
+
                 rf, ff = f.relpath(imdir).as_posix(), f.resolve()
                 attrs['文件'] = f'<a href="{ff}" target="_blank">{rf}</a><br/>'
                 # 如果要展示图片，可以加：<img src="{ff}" width=100/>，但实测效果不好
@@ -309,7 +349,6 @@ class Hesuan:
                 for col in self.columns:
                     row.append(attrs.get(col, ''))
                 ls.append(row)
-
         else:
             img_lists = self.get_img_fileslist(imdir, self.file_num)
             total_number = len(img_lists)
@@ -358,10 +397,19 @@ class Hesuan:
 
         def set_color(m):
             s1, s2 = m.groups()
+
+            parts = s1.split('-')
+            if len(parts) > 1:
+                parts[1] = f'{min(12, int(parts[1])):02}'
+            s1 = '-'.join(parts)
+
             # 每个日期颜色是固定的，而不是按照相距今天的天数来选的
-            i = (datetime.date.fromisoformat(s1) - datetime.date(2022, 1, 1)).days
-            c = colors[i % len(colors)]
-            return f'<td bgcolor="#{c}">{s1}{s2}</td>'
+            try:
+                i = (datetime.date.fromisoformat(s1) - datetime.date(2022, 1, 1)).days
+                c = colors[i % len(colors)]
+                return f'<td bgcolor="#{c}">{s1}{s2}</td>'
+            except ValueError:
+                return f'<td>{s1}{s2}</td>'
 
         res = df.to_html(escape=False)
         res = re.sub(r'<td>(\d{4}-\d{2}-\d{2})(.*?)</td>', set_color, res)
@@ -409,7 +457,7 @@ class MainWindow(QMainWindow):
         # 1 窗口核心属性
         self.setMinimumSize(QSize(900, 300))
         self.setWindowIcon(QIcon('models/xmut.jpg'))
-        self.setWindowTitle("核酸检测分析 @(厦门理工学院)福建省模式识别与图像理解重点实验室")
+        self.setWindowTitle("核酸检测分析v4.0（在线版） @(厦门理工学院)福建省模式识别与图像理解重点实验室")
 
         # 2 使用网格布局
         centralWidget = QWidget(self)
@@ -571,6 +619,7 @@ class MainWindow(QMainWindow):
         res = hs.parse(self.srcdir, self.get_students(), pb=self.pb)
         dstfile = self.dstfile
         dstfile.write_text(res, encoding='utf8')
+        hs.ppocr.write_cache()
         os.startfile(dstfile)
 
 
@@ -582,6 +631,18 @@ def gui():
     sys.exit(app.exec_())
 
 
+def 性能测试():
+    model = PPOCR()
+
+    files = list(XlPath('/home/chenkunze/data/hesuan/data/A1 核酸报告-按模板分类').rglob_images())
+    tt = time.time()
+    for f in tqdm(files):
+        model.parse_light(f)
+    print((time.time() - tt) / len(files), '秒/张')
+
+    model.write_cache()
+
+
 if __name__ == '__main__':
     if len(sys.argv) == 1:  # 默认执行main函数
         sys.argv += ['gui']
